/* eslint-env node */
module.exports = ({ getCard, getLinkedCardUserList, _ }) => {
  const AtUserPattern = /<at id="(\d+)"\/>/g
  const at = id => `<at id="${id}"/>`

  function getAtUserCardList(content, env) {
    // 兼容代骰，不完全严谨
    if (env.realUser.userId !== env.userId) {
      content = (content || '') + at(env.userId)
    }
    // 获取所有 @ 成员 id
    const ids = content ? Array.from(content.matchAll(AtUserPattern)).map(i => i[1]) : getLinkedCardUserList(env)
    const uniqIds = Array.from(new Set(ids))
    const userCards = uniqIds
      .map(userId => ({ userId, card: getCard({ ...env, userId }) }))
      .filter(({ card }) => card && card.type === 'coc')
    return userCards
  }

  function findLowestLuck(userCards) {
    const ret = []
    let lowest = Infinity
    userCards.forEach(item => {
      const luck = item.card.getEntry('LUCK').value // 前面保证了 coc 人物卡就一定存在
      item.luck = luck // 缓存一下获取结果，用于 luck 的列表展示
      if (luck < lowest) {
        lowest = luck
        ret.length = 0
        ret.push(item)
      } else if (luck === lowest) {
        ret.push(item)
      }
    })
    return [ret, lowest]
  }

  const successLevels = Object.freeze({
    '-2': '大失败',
    '-1': '失败',
    '1': '成功',
    '2': '困难成功',
    '3': '极难成功',
    '4': '大成功'
  })

  function vs(list, round) {
    list.forEach(item => {
      const roll = _.random(1, 100)
      const target = item.luck
      item.vs = { roll, target, level: decideByCoc0(roll, target) }
    })
    // 组装 result
    const lead = round === 1 ? '对抗结果：' : `第${round}轮对抗结果：`
    const body = list.map(item => `[${item.card.name}] ${item.vs.roll}/${item.vs.target} ${successLevels[item.vs.level]}`).join('\n')
    return lead + '\n' + body + '\n'
  }

  function decideByCoc0(roll, baseValue, targetValue = baseValue) {
    if ((baseValue < 50 && roll > 95) || (baseValue >= 50 && roll === 100)) return -2
    if (roll === 1) return 4
    if (roll > targetValue) return -1
    if (roll <= targetValue && roll <= baseValue / 5) return 3
    if (roll <= targetValue && roll <= baseValue / 2) return 2
    return 1
  }

  function deciderWinner(list) {
    const orderedList = _.orderBy(list, ['vs.level', 'vs.roll'], ['desc', 'asc']) // 本场景 target 肯定一样
    const winners = orderedList.filter(item => _.isEqual(item.vs, orderedList[0].vs))
    return winners
  }

  return {
    id: 'io.paotuan.plugin.coc.ttxy',
    name: 'COC 团体幸运检定',
    version: 4,
    customReply: [
      {
        id: 'ttxy',
        name: 'COC 团体幸运检定',
        description: '/ttxy @aaa @bbb 指定成员中幸运最低者进行幸运检定。若多人最低则随机选取一人。省略 @ 则从所有关联了 COC 人物卡的成员中选取幸运最低者',
        command: '^\\s*ttxy\\s*(?<content>.*)',
        trigger: 'regex',
        handler(env, { content }) {
          const userCards = getAtUserCardList(content, env)
          if (userCards.length === 0) {
            return '没有关联 COC 人物卡的成员'
          }
          // 获取幸运最低者
          const [options] = findLowestLuck(userCards)
          const luckList = userCards.map(item => `${item.card.name} [${item.luck}]`).join('\n')
          // 如有多人数值相同，则进行对抗直到分出胜负
          let winners = options
          let vsResult = ''
          let round = 1
          while (winners.length > 1) {
            vsResult += vs(winners, round++)
            winners = deciderWinner(winners)
          }
          const target = winners[0]
          // reply
          return `团体幸运列表：\n${luckList}\n${vsResult}本次团体幸运检定由 [${target.card.name}] 进行检定`
        }
      }
    ]
  }
}
